pub struct Uint<const BITS: usize, const LIMBS: usize> { /* private fields */ }Expand description
The ring of numbers modulo $2^{\mathtt{BITS}}$.
Uint implements nearly all traits and methods from the std unsigned
integer types, including most nightly only ones.
Notable differences from std uint types.
- The operators
+,-,*, etc. using wrapping math by default. The std operators panic on overflow in debug, and are undefined in release, see reference. - The
Uint::checked_shl,Uint::overflowing_shl, etc return overflow when non-zero bits are shifted out. In std they return overflow when the shift amount is greater than the bit size. - Some methods like
u64::div_euclidandu64::rem_euclidare left out because they are meaningless or redundant for unsigned integers. Std has them for compatibility with their signed integers. - Many functions that are
constin std are not inUint. Uint::to_le_bytesandUint::to_be_bytesrequire the output size to be provided as a const-generic argument. They will runtime panic if the provided size is incorrect.Uint::widening_multakes as argument anUintof arbitrary size and returns a result that is sized to fit the product without overflow (i.e. the sum of the bit sizes of self and the argument). The std version requires same-sized arguments and returns a pair of lower and higher bits.
Implementations
Sourceimpl<const BITS: usize, const LIMBS: usize> Uint<BITS, LIMBS>
impl<const BITS: usize, const LIMBS: usize> Uint<BITS, LIMBS>
Sourcepub fn abs_diff(self, other: Self) -> Self
pub fn abs_diff(self, other: Self) -> Self
Computes the absolute difference between self and other.
Returns $\left\vert \mathtt{self} - \mathtt{other} \right\vert$.
Sourcepub const fn checked_add(self, rhs: Self) -> Option<Self>
pub const fn checked_add(self, rhs: Self) -> Option<Self>
Computes self + rhs, returning None if overflow occurred.
Sourcepub const fn checked_neg(self) -> Option<Self>
pub const fn checked_neg(self) -> Option<Self>
Computes -self, returning None unless self == 0.
Sourcepub const fn checked_sub(self, rhs: Self) -> Option<Self>
pub const fn checked_sub(self, rhs: Self) -> Option<Self>
Computes self - rhs, returning None if overflow occurred.
Sourcepub const fn overflowing_add(self, rhs: Self) -> (Self, bool)
pub const fn overflowing_add(self, rhs: Self) -> (Self, bool)
Calculates $\mod{\mathtt{self} + \mathtt{rhs}}_{2^{BITS}}$.
Returns a tuple of the addition along with a boolean indicating whether an arithmetic overflow would occur. If an overflow would have occurred then the wrapped value is returned.
Sourcepub const fn overflowing_neg(self) -> (Self, bool)
pub const fn overflowing_neg(self) -> (Self, bool)
Calculates $\mod{-\mathtt{self}}_{2^{BITS}}$.
Returns !self + 1 using wrapping operations to return the value that
represents the negation of this unsigned value. Note that for positive
unsigned values overflow always occurs, but negating 0 does not
overflow.
Sourcepub const fn overflowing_sub(self, rhs: Self) -> (Self, bool)
pub const fn overflowing_sub(self, rhs: Self) -> (Self, bool)
Calculates $\mod{\mathtt{self} - \mathtt{rhs}}_{2^{BITS}}$.
Returns a tuple of the subtraction along with a boolean indicating whether an arithmetic overflow would occur. If an overflow would have occurred then the wrapped value is returned.
Sourcepub const fn saturating_add(self, rhs: Self) -> Self
pub const fn saturating_add(self, rhs: Self) -> Self
Computes self + rhs, saturating at the numeric bounds instead of
overflowing.
Sourcepub const fn saturating_sub(self, rhs: Self) -> Self
pub const fn saturating_sub(self, rhs: Self) -> Self
Computes self - rhs, saturating at the numeric bounds instead of
overflowing
Sourcepub const fn wrapping_add(self, rhs: Self) -> Self
pub const fn wrapping_add(self, rhs: Self) -> Self
Computes self + rhs, wrapping around at the boundary of the type.
Sourcepub const fn wrapping_neg(self) -> Self
pub const fn wrapping_neg(self) -> Self
Computes -self, wrapping around at the boundary of the type.
Sourcepub const fn wrapping_sub(self, rhs: Self) -> Self
pub const fn wrapping_sub(self, rhs: Self) -> Self
Computes self - rhs, wrapping around at the boundary of the type.
Sourceimpl<const BITS: usize, const LIMBS: usize> Uint<BITS, LIMBS>
impl<const BITS: usize, const LIMBS: usize> Uint<BITS, LIMBS>
Sourcepub fn to_base_le(&self, base: u64) -> impl Iterator<Item = u64>
pub fn to_base_le(&self, base: u64) -> impl Iterator<Item = u64>
Returns an iterator over the base base digits of the number in
little-endian order.
Pro tip: instead of setting base = 10, set it to the highest
power of 10 that still fits u64. This way much fewer iterations
are required to extract all the digits.
Panics
Panics if the base is less than 2.
Sourcepub fn to_base_be(&self, base: u64) -> impl Iterator<Item = u64>
pub fn to_base_be(&self, base: u64) -> impl Iterator<Item = u64>
Returns an iterator over the base base digits of the number in
big-endian order.
Pro tip: instead of setting base = 10, set it to the highest
power of 10 that still fits u64. This way much fewer iterations
are required to extract all the digits.
Use to_base_be_2 to extract the maximum number
of digits at once more efficiently.
Panics
Panics if the base is less than 2.
Examples
let n = ruint::aliases::U64::from(1234);
assert_eq!(n.to_base_be(10).collect::<Vec<_>>(), [1, 2, 3, 4]);
assert_eq!(n.to_base_be(1000000).collect::<Vec<_>>(), [1234]);
// `to_base_be_2` always returns digits maximally packed into `u64`s.
assert_eq!(n.to_base_be_2(10).collect::<Vec<_>>(), [1234]);
assert_eq!(n.to_base_be_2(1000000).collect::<Vec<_>>(), [1234]);Sourcepub fn to_base_be_2(&self, base: u64) -> impl Iterator<Item = u64>
pub fn to_base_be_2(&self, base: u64) -> impl Iterator<Item = u64>
Returns an iterator over the base base digits of the number in
big-endian order.
Always returns digits maximally packed into u64s.
Unlike to_base_be, this method:
- never heap-allocates memory, so it’s always faster
- always returns digits maximally packed into
u64s, so passing the constant base like2,8, instead of the highest power that fits in u64 is not needed
Panics
Panics if the base is less than 2.
Examples
See to_base_be.
Sourcepub fn from_base_le<I>(base: u64, digits: I) -> Result<Self, BaseConvertError>where
I: IntoIterator<Item = u64>,
pub fn from_base_le<I>(base: u64, digits: I) -> Result<Self, BaseConvertError>where
I: IntoIterator<Item = u64>,
Constructs the Uint from digits in the base base in little-endian.
Errors
BaseConvertError::InvalidBaseif the base is less than 2.BaseConvertError::InvalidDigitif a digit is out of range.BaseConvertError::Overflowif the number is too large to fit.
Sourcepub fn from_base_be<I: IntoIterator<Item = u64>>(
base: u64,
digits: I,
) -> Result<Self, BaseConvertError>
pub fn from_base_be<I: IntoIterator<Item = u64>>( base: u64, digits: I, ) -> Result<Self, BaseConvertError>
Constructs the Uint from digits in the base base in big-endian.
Errors
BaseConvertError::InvalidBaseif the base is less than 2.BaseConvertError::InvalidDigitif a digit is out of range.BaseConvertError::Overflowif the number is too large to fit.
Sourceimpl<const BITS: usize, const LIMBS: usize> Uint<BITS, LIMBS>
impl<const BITS: usize, const LIMBS: usize> Uint<BITS, LIMBS>
Sourcepub const fn bit(&self, index: usize) -> bool
pub const fn bit(&self, index: usize) -> bool
Returns whether a specific bit is set.
Returns false if index exceeds the bit width of the number.
Sourcepub const fn byte(&self, index: usize) -> u8
pub const fn byte(&self, index: usize) -> u8
Returns a specific byte. The byte at index 0 is the least significant
byte (little endian).
Panics
Panics if index is greater than or equal to the byte width of the
number.
Examples
let x = uint!(0x1234567890_U64);
let bytes = [
x.byte(0), // 0x90
x.byte(1), // 0x78
x.byte(2), // 0x56
x.byte(3), // 0x34
x.byte(4), // 0x12
x.byte(5), // 0x00
x.byte(6), // 0x00
x.byte(7), // 0x00
];
assert_eq!(bytes, x.to_le_bytes());Panics if out of range.
let x = uint!(0x1234567890_U64);
let _ = x.byte(8);Sourcepub const fn checked_byte(&self, index: usize) -> Option<u8>
pub const fn checked_byte(&self, index: usize) -> Option<u8>
Returns a specific byte, or None if index is out of range. The byte
at index 0 is the least significant byte (little endian).
Examples
let x = uint!(0x1234567890_U64);
assert_eq!(x.checked_byte(0), Some(0x90));
assert_eq!(x.checked_byte(7), Some(0x00));
// Out of range
assert_eq!(x.checked_byte(8), None);Sourcepub fn reverse_bits(self) -> Self
pub fn reverse_bits(self) -> Self
Reverses the order of bits in the integer. The least significant bit becomes the most significant bit, second least-significant bit becomes second most-significant bit, etc.
Sourcepub const fn leading_zeros(&self) -> usize
pub const fn leading_zeros(&self) -> usize
Returns the number of leading zeros in the binary representation of
self.
Sourcepub const fn leading_ones(&self) -> usize
pub const fn leading_ones(&self) -> usize
Returns the number of leading ones in the binary representation of
self.
Sourcepub fn trailing_zeros(&self) -> usize
pub fn trailing_zeros(&self) -> usize
Returns the number of trailing zeros in the binary representation of
self.
Sourcepub fn trailing_ones(&self) -> usize
pub fn trailing_ones(&self) -> usize
Returns the number of trailing ones in the binary representation of
self.
Sourcepub const fn count_ones(&self) -> usize
pub const fn count_ones(&self) -> usize
Returns the number of ones in the binary representation of self.
Sourcepub const fn count_zeros(&self) -> usize
pub const fn count_zeros(&self) -> usize
Returns the number of zeros in the binary representation of self.
Sourcepub const fn bit_len(&self) -> usize
pub const fn bit_len(&self) -> usize
Returns the dynamic length of this number in bits, ignoring leading zeros.
For the maximum length of the type, use Uint::BITS.
Sourcepub const fn byte_len(&self) -> usize
pub const fn byte_len(&self) -> usize
Returns the dynamic length of this number in bytes, ignoring leading zeros.
For the maximum length of the type, use Uint::BYTES.
Sourcepub fn most_significant_bits(&self) -> (u64, usize)
pub fn most_significant_bits(&self) -> (u64, usize)
Returns the most significant 64 bits of the number and the exponent.
Given return value $(\mathtt{bits}, \mathtt{exponent})$, the self can
be approximated as
$$ \mathtt{self} ≈ \mathtt{bits} ⋅ 2^\mathtt{exponent} $$
If self is $<≥> 2^{63}$, then exponent will be zero and bits will
have leading zeros.
Sourcepub const fn checked_shl(self, rhs: usize) -> Option<Self>
pub const fn checked_shl(self, rhs: usize) -> Option<Self>
Checked left shift by rhs bits.
Returns $\mathtt{self} ⋅ 2^{\mathtt{rhs}}$ or None if the result
would $≥ 2^{\mathtt{BITS}}$. That is, it returns None if the bits
shifted out would be non-zero.
Note: This differs from u64::checked_shl which returns None if the
shift is larger than BITS (which is IMHO not very useful).
Sourcepub const fn saturating_shl(self, rhs: usize) -> Self
pub const fn saturating_shl(self, rhs: usize) -> Self
Sourcepub const fn overflowing_shl(self, rhs: usize) -> (Self, bool)
pub const fn overflowing_shl(self, rhs: usize) -> (Self, bool)
Left shift by rhs bits with overflow detection.
Returns $\mod{\mathtt{value} ⋅ 2^{\mathtt{rhs}}}_{2^{\mathtt{BITS}}}$.
If the product is $≥ 2^{\mathtt{BITS}}$ it returns true. That is, it
returns true if the bits shifted out are non-zero.
Note: This differs from u64::overflowing_shl which returns true if
the shift is larger than BITS (which is IMHO not very useful).
Sourcepub const fn wrapping_shl(self, rhs: usize) -> Self
pub const fn wrapping_shl(self, rhs: usize) -> Self
Left shift by rhs bits.
Returns $\mod{\mathtt{value} ⋅ 2^{\mathtt{rhs}}}_{2^{\mathtt{BITS}}}$.
Note: This differs from u64::wrapping_shl which first reduces rhs
by BITS (which is IMHO not very useful).
Sourcepub const fn checked_shr(self, rhs: usize) -> Option<Self>
pub const fn checked_shr(self, rhs: usize) -> Option<Self>
Checked right shift by rhs bits.
$$ \frac{\mathtt{self}}{2^{\mathtt{rhs}}} $$
Returns the above or None if the division is not exact. This is the
same as
Note: This differs from u64::checked_shr which returns None if the
shift is larger than BITS (which is IMHO not very useful).
Sourcepub const fn overflowing_shr(self, rhs: usize) -> (Self, bool)
pub const fn overflowing_shr(self, rhs: usize) -> (Self, bool)
Right shift by rhs bits with underflow detection.
$$ \floor{\frac{\mathtt{self}}{2^{\mathtt{rhs}}}} $$
Returns the above and false if the division was exact, and true if
it was rounded down. This is the same as non-zero bits being shifted
out.
Note: This differs from u64::overflowing_shr which returns true if
the shift is larger than BITS (which is IMHO not very useful).
Sourcepub const fn wrapping_shr(self, rhs: usize) -> Self
pub const fn wrapping_shr(self, rhs: usize) -> Self
Right shift by rhs bits.
$$ \mathtt{wrapping\_shr}(\mathtt{self}, \mathtt{rhs}) = \floor{\frac{\mathtt{self}}{2^{\mathtt{rhs}}}} $$
Note: This differs from u64::wrapping_shr which first reduces rhs
by BITS (which is IMHO not very useful).
Sourcepub const fn arithmetic_shr(self, rhs: usize) -> Self
pub const fn arithmetic_shr(self, rhs: usize) -> Self
Arithmetic shift right by rhs bits.
Sourcepub const fn rotate_left(self, rhs: usize) -> Self
pub const fn rotate_left(self, rhs: usize) -> Self
Shifts the bits to the left by a specified amount, rhs, wrapping the
truncated bits to the end of the resulting integer.
Sourcepub const fn rotate_right(self, rhs: usize) -> Self
pub const fn rotate_right(self, rhs: usize) -> Self
Shifts the bits to the right by a specified amount, rhs, wrapping the
truncated bits to the beginning of the resulting integer.
Sourceimpl<const BITS: usize, const LIMBS: usize> Uint<BITS, LIMBS>
impl<const BITS: usize, const LIMBS: usize> Uint<BITS, LIMBS>
Sourcepub const BYTES: usize
pub const BYTES: usize
The size of this integer type in bytes. Note that some bits may be forced zero if BITS is not cleanly divisible by eight.
Sourcepub const fn as_le_slice(&self) -> &[u8] ⓘ
Available on little-endian only.
pub const fn as_le_slice(&self) -> &[u8] ⓘ
Access the underlying store as a little-endian slice of bytes.
Only available on little-endian targets.
If BITS does not evenly divide 8, it is padded with zero bits in the
most significant position.
Sourcepub const unsafe fn as_le_slice_mut(&mut self) -> &mut [u8] ⓘ
Available on little-endian only.
pub const unsafe fn as_le_slice_mut(&mut self) -> &mut [u8] ⓘ
Sourcepub fn as_le_bytes(&self) -> Cow<'_, [u8]>
Available on crate feature alloc only.
pub fn as_le_bytes(&self) -> Cow<'_, [u8]>
alloc only.Access the underlying store as a little-endian bytes.
Uses an optimized implementation on little-endian targets.
Sourcepub fn as_le_bytes_trimmed(&self) -> Cow<'_, [u8]>
Available on crate feature alloc only.
pub fn as_le_bytes_trimmed(&self) -> Cow<'_, [u8]>
alloc only.Access the underlying store as a little-endian bytes with trailing zeros removed.
Uses an optimized implementation on little-endian targets.
Sourcepub const fn to_le_bytes<const BYTES: usize>(&self) -> [u8; BYTES]
pub const fn to_le_bytes<const BYTES: usize>(&self) -> [u8; BYTES]
Converts the Uint to a little-endian byte array of size exactly
Self::BYTES.
Panics
Panics if the generic parameter BYTES is not exactly Self::BYTES.
Ideally this would be a compile time error, but this is blocked by
Rust issue #60551.
Sourcepub fn to_le_bytes_vec(&self) -> Vec<u8> ⓘ
Available on crate feature alloc only.
pub fn to_le_bytes_vec(&self) -> Vec<u8> ⓘ
alloc only.Converts the Uint to a little-endian byte vector of size exactly
Self::BYTES.
This method is useful when Self::to_le_bytes can not be used because
byte size is not known compile time.
Sourcepub fn to_le_bytes_trimmed_vec(&self) -> Vec<u8> ⓘ
Available on crate feature alloc only.
pub fn to_le_bytes_trimmed_vec(&self) -> Vec<u8> ⓘ
alloc only.Converts the Uint to a little-endian byte vector with trailing zeros
bytes removed.
Sourcepub const fn to_be_bytes<const BYTES: usize>(&self) -> [u8; BYTES]
pub const fn to_be_bytes<const BYTES: usize>(&self) -> [u8; BYTES]
Converts the Uint to a big-endian byte array of size exactly
Self::BYTES.
Panics
Panics if the generic parameter BYTES is not exactly Self::BYTES.
Ideally this would be a compile time error, but this is blocked by
Rust issue #60551.
Sourcepub fn to_be_bytes_vec(&self) -> Vec<u8> ⓘ
Available on crate feature alloc only.
pub fn to_be_bytes_vec(&self) -> Vec<u8> ⓘ
alloc only.Converts the Uint to a big-endian byte vector of size exactly
Self::BYTES.
This method is useful when Self::to_be_bytes can not be used because
byte size is not known compile time.
Sourcepub fn to_be_bytes_trimmed_vec(&self) -> Vec<u8> ⓘ
Available on crate feature alloc only.
pub fn to_be_bytes_trimmed_vec(&self) -> Vec<u8> ⓘ
alloc only.Converts the Uint to a big-endian byte vector with leading zeros
bytes removed.
Sourcepub const fn from_be_bytes<const BYTES: usize>(bytes: [u8; BYTES]) -> Self
pub const fn from_be_bytes<const BYTES: usize>(bytes: [u8; BYTES]) -> Self
Converts a big-endian byte array of size exactly
Self::BYTES to Uint.
Panics
Panics if the generic parameter BYTES is not exactly Self::BYTES.
Ideally this would be a compile time error, but this is blocked by
Rust issue #60551.
Panics if the value is too large for the bit-size of the Uint.
Sourcepub const fn from_be_slice(bytes: &[u8]) -> Self
pub const fn from_be_slice(bytes: &[u8]) -> Self
Creates a new integer from a big endian slice of bytes.
The slice is interpreted as a big endian number, and must be at most
Self::BYTES long.
Panics
Panics if the value is larger than fits the Uint.
Sourcepub const fn try_from_be_slice(bytes: &[u8]) -> Option<Self>
pub const fn try_from_be_slice(bytes: &[u8]) -> Option<Self>
Creates a new integer from a big endian slice of bytes.
The slice is interpreted as a big endian number, and must be at most
Self::BYTES long.
Sourcepub const fn from_le_bytes<const BYTES: usize>(bytes: [u8; BYTES]) -> Self
pub const fn from_le_bytes<const BYTES: usize>(bytes: [u8; BYTES]) -> Self
Converts a little-endian byte array of size exactly
Self::BYTES to Uint.
Panics
Panics if the generic parameter BYTES is not exactly Self::BYTES.
Ideally this would be a compile time error, but this is blocked by
Rust issue #60551.
Panics if the value is too large for the bit-size of the Uint.
Sourcepub const fn from_le_slice(bytes: &[u8]) -> Self
pub const fn from_le_slice(bytes: &[u8]) -> Self
Creates a new integer from a little endian slice of bytes.
The slice is interpreted as a little endian number, and must be at most
Self::BYTES long.
Panics
Panics if the value is larger than fits the Uint.
Sourcepub const fn try_from_le_slice(bytes: &[u8]) -> Option<Self>
pub const fn try_from_le_slice(bytes: &[u8]) -> Option<Self>
Creates a new integer from a little endian slice of bytes.
The slice is interpreted as a little endian number, and must be at most
Self::BYTES long.
Sourcepub fn copy_le_bytes_to(&self, buf: &mut [u8]) -> usize
pub fn copy_le_bytes_to(&self, buf: &mut [u8]) -> usize
Writes the little-endian representation of the Uint to the given
buffer. The buffer must be large enough to hold Self::BYTES bytes.
Panics
Panics if the buffer is not large enough to hold Self::BYTES bytes.
Returns
The number of bytes written to the buffer (always equal to
Self::BYTES, but often useful to make explicit for encoders).
Sourcepub fn checked_copy_le_bytes_to(&self, buf: &mut [u8]) -> Option<usize>
pub fn checked_copy_le_bytes_to(&self, buf: &mut [u8]) -> Option<usize>
Writes the little-endian representation of the Uint to the given
buffer. The buffer must be large enough to hold Self::BYTES bytes.
Returns
None, if the buffer is not large enough to hold Self::BYTES
bytes, and does not modify the buffer.
Some with the number of bytes written to the buffer (always
equal to Self::BYTES, but often useful to make explicit for
encoders).
Sourcepub fn copy_be_bytes_to(&self, buf: &mut [u8]) -> usize
pub fn copy_be_bytes_to(&self, buf: &mut [u8]) -> usize
Writes the big-endian representation of the Uint to the given
buffer. The buffer must be large enough to hold Self::BYTES bytes.
Panics
Panics if the buffer is not large enough to hold Self::BYTES bytes.
Returns
The number of bytes written to the buffer (always equal to
Self::BYTES, but often useful to make explicit for encoders).
Sourcepub fn checked_copy_be_bytes_to(&self, buf: &mut [u8]) -> Option<usize>
pub fn checked_copy_be_bytes_to(&self, buf: &mut [u8]) -> Option<usize>
Writes the big-endian representation of the Uint to the given
buffer. The buffer must be large enough to hold Self::BYTES bytes.
Returns
None, if the buffer is not large enough to hold Self::BYTES
bytes, and does not modify the buffer.
Some with the number of bytes written to the buffer (always
equal to Self::BYTES, but often useful to make explicit for
encoders).
Sourceimpl<const BITS: usize, const LIMBS: usize> Uint<BITS, LIMBS>
impl<const BITS: usize, const LIMBS: usize> Uint<BITS, LIMBS>
Sourcepub const fn const_is_zero(&self) -> bool
pub const fn const_is_zero(&self) -> bool
Returns true if the value is zero.
Note that this currently might perform worse than
is_zero.
Sourceimpl<const BITS: usize, const LIMBS: usize> Uint<BITS, LIMBS>
impl<const BITS: usize, const LIMBS: usize> Uint<BITS, LIMBS>
Sourcepub fn checked_div(self, rhs: Self) -> Option<Self>
pub fn checked_div(self, rhs: Self) -> Option<Self>
Computes self / rhs, returning None if rhs == 0.
Sourcepub fn checked_rem(self, rhs: Self) -> Option<Self>
pub fn checked_rem(self, rhs: Self) -> Option<Self>
Computes self % rhs, returning None if rhs == 0.
Sourcepub fn wrapping_div(self, rhs: Self) -> Self
pub fn wrapping_div(self, rhs: Self) -> Self
Sourcepub fn wrapping_rem(self, rhs: Self) -> Self
pub fn wrapping_rem(self, rhs: Self) -> Self
Sourceimpl<const BITS: usize, const LIMBS: usize> Uint<BITS, LIMBS>
impl<const BITS: usize, const LIMBS: usize> Uint<BITS, LIMBS>
Sourcepub fn from<T>(value: T) -> Selfwhere
Self: UintTryFrom<T>,
pub fn from<T>(value: T) -> Selfwhere
Self: UintTryFrom<T>,
Construct a new Uint from the value.
Panics
Panics if the conversion fails, for example if the value is too large
for the bit-size of the Uint. The panic will be attributed to the
call site.
Examples
assert_eq!(U8::from(142_u16), 142_U8);
assert_eq!(U64::from(0x7014b4c2d1f2_U256), 0x7014b4c2d1f2_U64);
assert_eq!(U64::from(3.145), 3_U64);Sourcepub fn saturating_from<T>(value: T) -> Selfwhere
Self: UintTryFrom<T>,
pub fn saturating_from<T>(value: T) -> Selfwhere
Self: UintTryFrom<T>,
Construct a new Uint from the value saturating the value to the
minimum or maximum value of the Uint.
If the value is not a number (like f64::NAN), then the result is
set zero.
Examples
assert_eq!(U8::saturating_from(300_u16), 255_U8);
assert_eq!(U8::saturating_from(-10_i16), 0_U8);
assert_eq!(U32::saturating_from(0x7014b4c2d1f2_U256), U32::MAX);Sourcepub fn wrapping_from<T>(value: T) -> Selfwhere
Self: UintTryFrom<T>,
pub fn wrapping_from<T>(value: T) -> Selfwhere
Self: UintTryFrom<T>,
Construct a new Uint from the value saturating the value to the
minimum or maximum value of the Uint.
If the value is not a number (like f64::NAN), then the result is
set zero.
Examples
assert_eq!(U8::wrapping_from(300_u16), 44_U8);
assert_eq!(U8::wrapping_from(-10_i16), 246_U8);
assert_eq!(U32::wrapping_from(0x7014b4c2d1f2_U256), 0xb4c2d1f2_U32);Sourcepub fn wrapping_to<T>(&self) -> Twhere
Self: UintTryTo<T>,
pub fn wrapping_to<T>(&self) -> Twhere
Self: UintTryTo<T>,
Examples
assert_eq!(300_U12.wrapping_to::<i8>(), 44_i8);
assert_eq!(255_U32.wrapping_to::<i8>(), -1_i8);
assert_eq!(0x1337cafec0d3_U256.wrapping_to::<U32>(), 0xcafec0d3_U32);Sourcepub fn saturating_to<T>(&self) -> Twhere
Self: UintTryTo<T>,
pub fn saturating_to<T>(&self) -> Twhere
Self: UintTryTo<T>,
Examples
assert_eq!(300_U12.saturating_to::<i16>(), 300_i16);
assert_eq!(255_U32.saturating_to::<i8>(), 127);
assert_eq!(0x1337cafec0d3_U256.saturating_to::<U32>(), U32::MAX);Sourceimpl<const BITS: usize, const LIMBS: usize> Uint<BITS, LIMBS>
impl<const BITS: usize, const LIMBS: usize> Uint<BITS, LIMBS>
Sourcepub fn gcd_extended(self, other: Self) -> (Self, Self, Self, bool)
pub fn gcd_extended(self, other: Self) -> (Self, Self, Self, bool)
⚠️ Compute the greatest common divisor and the Bézout coefficients.
Warning. This is API is unstable and may change in a minor release.
Returns $(\mathtt{gcd}, \mathtt{x}, \mathtt{y}, \mathtt{sign})$ such that
$$ \gcd(\mathtt{self}, \mathtt{other}) = \mathtt{gcd} = \begin{cases} \mathtt{self} · \mathtt{x} - \mathtt{other} . \mathtt{y} & \mathtt{sign} \\ \mathtt{other} · \mathtt{y} - \mathtt{self} · \mathtt{x} & ¬\mathtt{sign} \end{cases} $$
Note that the intermediate products may overflow, even though the result
after subtraction will fit in the bit size of the Uint.
Sourceimpl<const BITS: usize, const LIMBS: usize> Uint<BITS, LIMBS>
impl<const BITS: usize, const LIMBS: usize> Uint<BITS, LIMBS>
Sourcepub fn checked_log(self, base: Self) -> Option<usize>
Available on crate feature std only.
pub fn checked_log(self, base: Self) -> Option<usize>
std only.Returns the logarithm of the number, rounded down.
Returns None if the base is less than two, or this number is zero.
Sourcepub fn checked_log10(self) -> Option<usize>
Available on crate feature std only.
pub fn checked_log10(self) -> Option<usize>
std only.Returns the base 10 logarithm of the number, rounded down.
Returns None if the number is zero.
Sourcepub fn checked_log2(self) -> Option<usize>
Available on crate feature std only.
pub fn checked_log2(self) -> Option<usize>
std only.Returns the base 2 logarithm of the number, rounded down.
This is equivalent to the index of the highest set bit.
Returns None if the number is zero.
Sourcepub fn log(self, base: Self) -> usize
Available on crate feature std only.
pub fn log(self, base: Self) -> usize
std only.Returns the logarithm of the number, rounded down.
Panics
Panics if the base is less than 2 or if the number is zero.
Sourcepub fn log10(self) -> usize
Available on crate feature std only.
pub fn log10(self) -> usize
std only.Returns the base 10 logarithm of the number, rounded down.
Panics
Panics if the base if the number is zero.
Sourcepub fn log2(self) -> usize
Available on crate feature std only.
pub fn log2(self) -> usize
std only.Returns the base 2 logarithm of the number, rounded down.
Panics
Panics if the base if the number is zero.
Sourcepub fn approx_log(self, base: f64) -> f64
Available on crate feature std only.
pub fn approx_log(self, base: f64) -> f64
std only.Double precision logarithm.
Sourcepub fn approx_log2(self) -> f64
Available on crate feature std only.
pub fn approx_log2(self) -> f64
std only.Double precision binary logarithm.
Examples
assert_eq!(0_U64.approx_log2(), f64::NEG_INFINITY);
assert_eq!(1_U64.approx_log2(), 0.0);
assert_eq!(2_U64.approx_log2(), 1.0);
assert_eq!(U64::MAX.approx_log2(), 64.0);Sourcepub fn approx_log10(self) -> f64
Available on crate feature std only.
pub fn approx_log10(self) -> f64
std only.Double precision decimal logarithm.
Sourceimpl<const BITS: usize, const LIMBS: usize> Uint<BITS, LIMBS>
impl<const BITS: usize, const LIMBS: usize> Uint<BITS, LIMBS>
Sourcepub fn reduce_mod(self, modulus: Self) -> Self
pub fn reduce_mod(self, modulus: Self) -> Self
⚠️ Compute $\mod{\mathtt{self}}_{\mathtt{modulus}}$.
Warning. This function is not part of the stable API.
Returns zero if the modulus is zero.
Sourcepub fn add_mod(self, rhs: Self, modulus: Self) -> Self
pub fn add_mod(self, rhs: Self, modulus: Self) -> Self
Compute $\mod{\mathtt{self} + \mathtt{rhs}}_{\mathtt{modulus}}$.
Returns zero if the modulus is zero.
Sourcepub fn mul_mod(self, rhs: Self, modulus: Self) -> Self
pub fn mul_mod(self, rhs: Self, modulus: Self) -> Self
Compute $\mod{\mathtt{self} ⋅ \mathtt{rhs}}_{\mathtt{modulus}}$.
Returns zero if the modulus is zero.
See mul_redc for a faster variant at the cost of
some pre-computation.
Sourcepub fn pow_mod(self, exp: Self, modulus: Self) -> Self
pub fn pow_mod(self, exp: Self, modulus: Self) -> Self
Compute $\mod{\mathtt{self}^{\mathtt{rhs}}}_{\mathtt{modulus}}$.
Returns zero if the modulus is zero.
Sourcepub fn inv_mod(self, modulus: Self) -> Option<Self>
pub fn inv_mod(self, modulus: Self) -> Option<Self>
Compute $\mod{\mathtt{self}^{-1}}_{\mathtt{modulus}}$.
Returns None if the inverse does not exist.
Sourcepub fn mul_redc(self, other: Self, modulus: Self, inv: u64) -> Self
pub fn mul_redc(self, other: Self, modulus: Self, inv: u64) -> Self
Montgomery multiplication.
Requires self and other to be less than modulus.
Computes
$$ \mod{\frac{\mathtt{self} ⋅ \mathtt{other}}{ 2^{64 · \mathtt{LIMBS}}}}_{\mathtt{modulus}} $$
This is useful because it can be computed notably faster than
mul_mod. Many computations can be done by
pre-multiplying values with $R = 2^{64 · \mathtt{LIMBS}}$
and then using mul_redc instead of
mul_mod.
For this algorithm to work, it needs an extra parameter inv which must
be set to
$$ \mathtt{inv} = \mod{\frac{-1}{\mathtt{modulus}} }_{2^{64}} $$
The inv value only exists for odd values of modulus. It can be
computed using inv_ring from U64.
let inv = U64::wrapping_from(modulus).inv_ring().unwrap().wrapping_neg().to();
let prod = 5_U256.mul_redc(6_U256, modulus, inv);Panics
Panics if inv is not correct in debug mode.
Sourcepub fn square_redc(self, modulus: Self, inv: u64) -> Self
pub fn square_redc(self, modulus: Self, inv: u64) -> Self
Montgomery squaring.
See Self::mul_redc.
Sourceimpl<const BITS: usize, const LIMBS: usize> Uint<BITS, LIMBS>
impl<const BITS: usize, const LIMBS: usize> Uint<BITS, LIMBS>
Sourcepub fn checked_mul(self, rhs: Self) -> Option<Self>
pub fn checked_mul(self, rhs: Self) -> Option<Self>
Computes self * rhs, returning None if overflow occurred.
Sourcepub fn overflowing_mul(self, rhs: Self) -> (Self, bool)
pub fn overflowing_mul(self, rhs: Self) -> (Self, bool)
Calculates the multiplication of self and rhs.
Returns a tuple of the multiplication along with a boolean indicating whether an arithmetic overflow would occur. If an overflow would have occurred then the wrapped value is returned.
Examples
assert_eq!(1_U1.overflowing_mul(1_U1), (1_U1, false));
assert_eq!(
0x010000000000000000_U65.overflowing_mul(0x010000000000000000_U65),
(0x000000000000000000_U65, true)
);Sourcepub fn saturating_mul(self, rhs: Self) -> Self
pub fn saturating_mul(self, rhs: Self) -> Self
Computes self * rhs, saturating at the numeric bounds instead of
overflowing.
Sourcepub fn wrapping_mul(self, rhs: Self) -> Self
pub fn wrapping_mul(self, rhs: Self) -> Self
Computes self * rhs, wrapping around at the boundary of the type.
Sourcepub fn inv_ring(self) -> Option<Self>
pub fn inv_ring(self) -> Option<Self>
Computes the inverse modulo $2^{\mathtt{BITS}}$ of self, returning
None if the inverse does not exist.
Sourcepub fn widening_mul<const BITS_RHS: usize, const LIMBS_RHS: usize, const BITS_RES: usize, const LIMBS_RES: usize>(
self,
rhs: Uint<BITS_RHS, LIMBS_RHS>,
) -> Uint<BITS_RES, LIMBS_RES>
pub fn widening_mul<const BITS_RHS: usize, const LIMBS_RHS: usize, const BITS_RES: usize, const LIMBS_RES: usize>( self, rhs: Uint<BITS_RHS, LIMBS_RHS>, ) -> Uint<BITS_RES, LIMBS_RES>
Calculates the complete product self * rhs without the possibility to
overflow.
The argument rhs can be any size Uint, the result size is the sum
of the bit-sizes of self and rhs.
Panics
This function will runtime panic of the const generic arguments are incorrect.
Examples
assert_eq!(0_U0.widening_mul(0_U0), 0_U0);
assert_eq!(1_U1.widening_mul(1_U1), 1_U2);
assert_eq!(3_U2.widening_mul(7_U3), 21_U5);Sourceimpl<const BITS: usize, const LIMBS: usize> Uint<BITS, LIMBS>
impl<const BITS: usize, const LIMBS: usize> Uint<BITS, LIMBS>
Sourcepub fn checked_pow(self, exp: Self) -> Option<Self>
pub fn checked_pow(self, exp: Self) -> Option<Self>
Raises self to the power of exp.
Returns None if the result would overflow.
Sourcepub fn overflowing_pow(self, exp: Self) -> (Self, bool)
pub fn overflowing_pow(self, exp: Self) -> (Self, bool)
Raises self to the power of exp and if the result would overflow.
Examples
assert_eq!(
36_U64.overflowing_pow(12_U64),
(0x41c21cb8e1000000_U64, false)
);
assert_eq!(
36_U64.overflowing_pow(13_U64),
(0x3f4c09ffa4000000_U64, true)
);
assert_eq!(
36_U68.overflowing_pow(13_U68),
(0x093f4c09ffa4000000_U68, false)
);
assert_eq!(16_U65.overflowing_pow(32_U65), (0_U65, true));Small cases:
assert_eq!(0_U0.overflowing_pow(0_U0), (0_U0, false));
assert_eq!(0_U1.overflowing_pow(0_U1), (1_U1, false));
assert_eq!(0_U1.overflowing_pow(1_U1), (0_U1, false));
assert_eq!(1_U1.overflowing_pow(0_U1), (1_U1, false));
assert_eq!(1_U1.overflowing_pow(1_U1), (1_U1, false));Sourcepub fn pow(self, exp: Self) -> Self
pub fn pow(self, exp: Self) -> Self
Raises self to the power of exp, wrapping around on overflow.
Sourcepub fn saturating_pow(self, exp: Self) -> Self
pub fn saturating_pow(self, exp: Self) -> Self
Raises self to the power of exp, saturating on overflow.
Sourcepub fn wrapping_pow(self, exp: Self) -> Self
pub fn wrapping_pow(self, exp: Self) -> Self
Raises self to the power of exp, wrapping around on overflow.
Sourcepub fn approx_pow2(exp: f64) -> Option<Self>
Available on crate feature std only.
pub fn approx_pow2(exp: f64) -> Option<Self>
std only.Construct from double precision binary logarithm.
Examples
assert_eq!(U64::approx_pow2(-2.0), Some(0_U64));
assert_eq!(U64::approx_pow2(-1.0), Some(1_U64));
assert_eq!(U64::approx_pow2(0.0), Some(1_U64));
assert_eq!(U64::approx_pow2(1.0), Some(2_U64));
assert_eq!(U64::approx_pow2(1.6), Some(3_U64));
assert_eq!(U64::approx_pow2(2.0), Some(4_U64));
assert_eq!(U64::approx_pow2(64.0), None);
assert_eq!(U64::approx_pow2(10.385), Some(1337_U64));Sourceimpl<const BITS: usize, const LIMBS: usize> Uint<BITS, LIMBS>
impl<const BITS: usize, const LIMBS: usize> Uint<BITS, LIMBS>
Sourcepub fn root(self, degree: usize) -> Self
Available on crate feature std only.
pub fn root(self, degree: usize) -> Self
std only.Computes the floor of the degree-th root of the number.
$$ \floor{\sqrt[\mathtt{degree}]{\mathtt{self}}} $$
Panics
Panics if degree is zero.
Examples
assert_eq!(0_U64.root(2), 0_U64);
assert_eq!(1_U64.root(63), 1_U64);
assert_eq!(0x0032da8b0f88575d_U63.root(64), 1_U63);
assert_eq!(0x1756800000000000_U63.root(34), 3_U63);Sourceimpl<const BITS: usize, const LIMBS: usize> Uint<BITS, LIMBS>
impl<const BITS: usize, const LIMBS: usize> Uint<BITS, LIMBS>
Sourcepub const fn is_power_of_two(self) -> bool
pub const fn is_power_of_two(self) -> bool
Returns true if and only if self == 2^k for some k.
Sourcepub const fn next_power_of_two(self) -> Self
pub const fn next_power_of_two(self) -> Self
Returns the smallest power of two greater than or equal to self.
Panics
Panics if the value overlfows.
Sourcepub const fn checked_next_power_of_two(self) -> Option<Self>
pub const fn checked_next_power_of_two(self) -> Option<Self>
Returns the smallest power of two greater than or equal to self. If
the next power of two is greater than the type’s maximum value,
None is returned, otherwise the power of two is wrapped in
Some.
Examples
assert_eq!(0_U64.checked_next_power_of_two(), Some(1_U64));
assert_eq!(1_U64.checked_next_power_of_two(), Some(1_U64));
assert_eq!(2_U64.checked_next_power_of_two(), Some(2_U64));
assert_eq!(3_U64.checked_next_power_of_two(), Some(4_U64));
assert_eq!(U64::MAX.checked_next_power_of_two(), None);Sourcepub fn next_multiple_of(self, rhs: Self) -> Self
pub fn next_multiple_of(self, rhs: Self) -> Self
Calculates the smallest value greater than or equal to self that is a multiple of rhs.
Panics
This function will panic if rhs is 0 or the operation results in
overflow.
Sourcepub fn checked_next_multiple_of(self, rhs: Self) -> Option<Self>
pub fn checked_next_multiple_of(self, rhs: Self) -> Option<Self>
Calculates the smallest value greater than or equal to self that is a
multiple of rhs. Returns None is rhs is zero or the
operation would result in overflow.
Examples
assert_eq!(16_U64.checked_next_multiple_of(8_U64), Some(16_U64));
assert_eq!(23_U64.checked_next_multiple_of(8_U64), Some(24_U64));
assert_eq!(1_U64.checked_next_multiple_of(0_U64), None);
assert_eq!(U64::MAX.checked_next_multiple_of(2_U64), None);assert_eq!(0_U0.checked_next_multiple_of(0_U0), None);
assert_eq!(0_U1.checked_next_multiple_of(0_U1), None);
assert_eq!(0_U1.checked_next_multiple_of(1_U1), Some(0_U1));
assert_eq!(1_U1.checked_next_multiple_of(0_U1), None);
assert_eq!(1_U1.checked_next_multiple_of(1_U1), Some(1_U1));Sourceimpl<const BITS: usize, const LIMBS: usize> Uint<BITS, LIMBS>
impl<const BITS: usize, const LIMBS: usize> Uint<BITS, LIMBS>
Sourcepub fn from_str_radix(src: &str, radix: u64) -> Result<Self, ParseError>
pub fn from_str_radix(src: &str, radix: u64) -> Result<Self, ParseError>
Parse a string into a Uint.
For bases 2 to 36, the case-agnostic alphabet 0—1, a—b is used and _
are ignored. For bases 37 to 64, the case-sensitive alphabet a—z, A—Z,
0—9, {+-}, {/,_} is used. That is, for base 64 it is compatible with
all the common base64 variants.
Errors
ParseError::InvalidDigitif the string contains a non-digit.ParseError::InvalidRadixif the radix is larger than 64.ParseError::BaseConvertErrorifUint::from_base_befails.
Sourceimpl<const BITS: usize, const LIMBS: usize> Uint<BITS, LIMBS>
impl<const BITS: usize, const LIMBS: usize> Uint<BITS, LIMBS>
Sourcepub fn random() -> Self
Available on crate features rand-09 and std only.
pub fn random() -> Self
rand-09 and std only.Creates a new Uint with the default cryptographic random number
generator.
This is currently rand::rng().
Sourcepub fn random_with<R: RngCore + ?Sized>(rng: &mut R) -> Self
Available on crate feature rand-09 only.
pub fn random_with<R: RngCore + ?Sized>(rng: &mut R) -> Self
rand-09 only.Creates a new Uint with the given random number generator.
Sourceimpl<const BITS: usize, const LIMBS: usize> Uint<BITS, LIMBS>
impl<const BITS: usize, const LIMBS: usize> Uint<BITS, LIMBS>
Sourcepub const ONE: Self
pub const ONE: Self
The value one. This is useful to have as a constant for use in const fn.
Zero if BITS is zero.
Sourcepub const MIN: Self = Self::ZERO
pub const MIN: Self = Self::ZERO
The smallest value that can be represented by this integer type.
Synonym for Self::ZERO.
Sourcepub const MAX: Self
pub const MAX: Self
The largest value that can be represented by this integer type, $2^{\mathtt{BITS}} − 1$.
Sourcepub const unsafe fn as_limbs_mut(&mut self) -> &mut [u64; LIMBS]
pub const unsafe fn as_limbs_mut(&mut self) -> &mut [u64; LIMBS]
Access the array of limbs.
Safety
This function is unsafe because it allows setting a bit outside the bit size if the bit-size is not limb-aligned.
Sourcepub const fn into_limbs(self) -> [u64; LIMBS]
pub const fn into_limbs(self) -> [u64; LIMBS]
Convert to a array of limbs.
Limbs are least significant first.
Sourcepub const fn from_limbs(limbs: [u64; LIMBS]) -> Self
pub const fn from_limbs(limbs: [u64; LIMBS]) -> Self
Construct a new integer from little-endian a array of limbs.
Panics
Panics it LIMBS is not equal to nlimbs(BITS).
Panics if the value is too large for the bit-size of the Uint.
Sourcepub fn from_limbs_slice(slice: &[u64]) -> Self
pub fn from_limbs_slice(slice: &[u64]) -> Self
Construct a new integer from little-endian a slice of limbs.
Panics
Panics if the value is too large for the bit-size of the Uint.
Sourcepub fn checked_from_limbs_slice(slice: &[u64]) -> Option<Self>
pub fn checked_from_limbs_slice(slice: &[u64]) -> Option<Self>
Construct a new integer from little-endian a slice of limbs, or None
if the value is too large for the Uint.
Sourcepub fn wrapping_from_limbs_slice(slice: &[u64]) -> Self
pub fn wrapping_from_limbs_slice(slice: &[u64]) -> Self
Construct a new Uint from a little-endian slice of limbs. Returns
a potentially truncated value.
Sourcepub fn overflowing_from_limbs_slice(slice: &[u64]) -> (Self, bool)
pub fn overflowing_from_limbs_slice(slice: &[u64]) -> (Self, bool)
Construct a new Uint from a little-endian slice of limbs. Returns
a potentially truncated value and a boolean indicating whether the value
was truncated.
Sourcepub fn saturating_from_limbs_slice(slice: &[u64]) -> Self
pub fn saturating_from_limbs_slice(slice: &[u64]) -> Self